@@ -1,5 +1,5 @@
 #  diceware -- passphrases to remember
-#  Copyright (C) 2015, 2016  Uli Fouquet
+#  Copyright (C) 2015-2018  Uli Fouquet
 #
 #  This program is free software: you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -18,10 +18,13 @@
 import argparse
 import pkg_resources
 import sys
+import logging
+from errno import ENOENT
 from random import SystemRandom
 from diceware.config import get_config_dict
+from diceware.logger import configure
 from diceware.wordlist import (
-    WordList, get_wordlist_path, WORDLISTS_DIR, get_wordlist_names,
+    WordList, get_wordlist_path, get_wordlists_dir, get_wordlist_names,
     )
 
 __version__ = pkg_resources.get_distribution('diceware').version
@@ -52,10 +55,11 @@ def print_version():
     """Output current version and other infos.
     """
     print("diceware %s" % __version__)
-    print("Copyright (C) 2015, 2016 Uli Fouquet")
+    print("Copyright (C) 2015-2018 Uli Fouquet")
     print("diceware is based on suggestions of Arnold G. Reinhold.")
     print("See http://diceware.com for details.")
-    print("'Diceware' is a trademark of Arnold G. Reinhold.")
+    print("'Diceware' is a trademark of Arnold G Reinhold,"
+          " used with permission")
     print(GPL_TEXT)
 
 
@@ -85,7 +89,7 @@ def handle_options(args):
     defaults = get_config_dict()
     parser = argparse.ArgumentParser(
         description="Create a passphrase",
-        epilog="Wordlists are stored in %s" % WORDLISTS_DIR
+        epilog="Wordlists are stored in %s" % get_wordlists_dir()
         )
     parser.add_argument(
         '-n', '--num', default=6, type=int,
@@ -110,12 +114,12 @@ def handle_options(args):
             "Get randomness from this source. Possible values: `%s'. "
             "Default: system" % "', `".join(sorted(random_sources))))
     parser.add_argument(
-        '-w', '--wordlist', default='en_securedrop', choices=wordlist_names,
+        '-w', '--wordlist', default='en_eff', choices=wordlist_names,
         metavar="NAME",
         help=(
             "Use words from this wordlist. Possible values: `%s'. "
             "Wordlists are stored in the folder displayed below. "
-            "Default: en_securedrop" % "', `".join(wordlist_names)))
+            "Default: en_eff" % "', `".join(wordlist_names)))
     realdice_group = parser.add_argument_group(
         "Arguments related to `realdice' randomsource",
         )
@@ -125,7 +129,6 @@ def handle_options(args):
         )
     parser.add_argument(
         'infile', nargs='?', metavar='INFILE', default=None,
-        type=argparse.FileType('r'),
         help="Input wordlist. `-' will read from stdin.",
         )
     parser.add_argument(
@@ -180,7 +183,7 @@ def get_passphrase(options=None):
     if options is None:
         options = handle_options(args=[])
     if options.infile is None:
-        options.infile = open(get_wordlist_path(options.wordlist), 'r')
+        options.infile = get_wordlist_path(options.wordlist)
     word_list = WordList(options.infile)
     rnd_source = get_random_sources()[options.randomsource]
     rnd = rnd_source(options)
@@ -204,7 +207,16 @@ def main(args=None):
     if args is None:
         args = sys.argv[1:]
     options = handle_options(args)
+    configure(options.verbose)
     if options.version:
         print_version()
         raise SystemExit(0)
-    print(get_passphrase(options))
+    try:
+        print(get_passphrase(options))
+    except (OSError, IOError) as infile_error:
+        if getattr(infile_error, 'errno', 0) == ENOENT:
+            logging.getLogger('ulif.diceware').error(
+                "The file '%s' does not exist." % infile_error.filename)
+            raise SystemExit(1)
+        else:
+            raise
