@@ -1,5 +1,5 @@
 #  diceware -- passphrases to remember
-#  Copyright (C) 2015, 2016  Uli Fouquet
+#  Copyright (C) 2015-2017  Uli Fouquet
 #
 #  This program is free software: you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -20,9 +20,9 @@ direct API calls.
 
 """
 try:
-    import configparser                  # Python 3.x
-except ImportError:                      # pragma: no cover
-    import ConfigParser as configparser  # Python 2.x
+    from configparser import ConfigParser as SafeParser      # Python 3.x
+except ImportError:                                         # pragma: no cover
+    from ConfigParser import SafeConfigParser as SafeParser  # Python 2.x
 import os
 
 
@@ -33,7 +33,7 @@ OPTIONS_DEFAULTS = dict(
     delimiter="",
     randomsource="system",
     verbose=0,
-    wordlist="en_securedrop",
+    wordlist="en_eff",
     dice_sides=6,
     )
 
@@ -57,7 +57,7 @@ def get_configparser(path_list=None):
     """
     if path_list is None:
         path_list = valid_locations()
-    parser = configparser.SafeConfigParser()
+    parser = SafeParser()
     found = parser.read(path_list)
     return found, parser
 
