@@ -1,5 +1,5 @@
 #  diceware -- passphrases to remember
-#  Copyright (C) 2016  Uli Fouquet and contributors
+#  Copyright (C) 2016-2018  Uli Fouquet and contributors
 #
 #  This program is free software: you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -43,6 +43,12 @@ except ImportError:  # NOQA  # pragma: no cover
         def createLock(self):
             pass
 
+        def acquire(self):
+            pass
+
+        def release(self):
+            pass
+
 
 #: Logger that can be used for all diceware related messages.
 logger = logging.getLogger("ulif.diceware")
@@ -50,14 +56,16 @@ logger.addHandler(NullHandler())
 
 
 def configure(verbosity=None):
-    """Configure global duceware logger.
+    """Configure global diceware logger.
 
     `verbosity` sets the diceware logger verbosity. ``0`` enables info
-    mode, while all numbes > 2 enable debug mode.
+    mode, while all numbers > 2 enable debug mode.
+
+    If no `verbosity` is given, we leave the logging level untouched.
     """
     if verbosity is None:
         return
-    logger.setLevel(logging.CRITICAL)
+    logger.setLevel(logging.ERROR)
     if verbosity == 1:
         logger.setLevel(logging.INFO)
     elif verbosity > 1:
