@@ -1,5 +1,5 @@
 #  diceware -- passphrases to remember
-#  Copyright (C) 2015, 2016  Uli Fouquet and contributors.
+#  Copyright (C) 2015-2017  Uli Fouquet and contributors.
 #
 #  This program is free software: you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -153,29 +153,36 @@ class RealDiceRandomSource(object):
                 num_rolls, num_rolls))
         return
 
+    def get_num_rolls(self, seq_len):
+        """Compute how many dice rolls we need to pick a value from a sequence
+        """
+        num_rolls = int(math.log(seq_len, self.dice_sides))
+        if num_rolls < 1:
+            # If this happens, there are less values in the sequence to
+            # choose from than there are dice sides.
+            num_rolls = 1
+        return num_rolls
+
     def choice(self, sequence):
         """Pick one item out of `sequence`.
         """
         if len(sequence) == 1:
             return sequence[0]  # no need to roll dice.
-        num_rolls = int(math.log(len(sequence), self.dice_sides))
-        if num_rolls < 1:
-            # If this happens, there are less values in the sequence to
-            # choose from than there are dice sides.
-            # Check whether len(sequence) is a factor of dice_sides
-            if self.dice_sides % len(sequence) == 0:
-                num_rolls = 1
-            else:
-                # otherwise We will perform one extra roll and apply modulo
-                num_rolls = 2
+        num_rolls = self.get_num_rolls(len(sequence))
         self.pre_check(num_rolls, sequence)
-        result = 0
-        for i in range(num_rolls, 0, -1):
-            rolled = None
-            while rolled not in [
-                    str(x) for x in range(1, self.dice_sides + 1)]:
-                rolled = input_func(
-                    "What number shows dice number %s? " % (num_rolls - i + 1))
-            result += ((self.dice_sides ** (i - 1)) * (int(rolled) - 1))
-            result = result % len(sequence)
+        repeat = True
+        while repeat:
+            result = 0
+            for i in range(num_rolls, 0, -1):
+                rolled = None
+                while rolled not in [
+                        str(x) for x in range(1, self.dice_sides + 1)]:
+                    rolled = input_func(
+                        "What number shows dice number %s? " % (
+                            num_rolls - i + 1))
+                result += ((self.dice_sides ** (i - 1)) * (int(rolled) - 1))
+            if result < len(sequence):
+                repeat = False
+            else:
+                print("Value out of range. Please roll dice again.")
         return sequence[result]
